@@ -52,23 +52,24 @@ using namespace std;
 class CrushWrapper {
   mutable Mutex mapper_lock;
 public:
+  struct crush_map *crush;
   std::map<int32_t, string> type_map; /* bucket/device type names */
   std::map<int32_t, string> name_map; /* bucket/device names */
   std::map<int32_t, string> rule_name_map;
 
-private:
-  struct crush_map *crush;
   /* reverse maps */
-  mutable bool have_rmaps;
-  mutable std::map<string, int> type_rmap, name_rmap, rule_name_rmap;
-  void build_rmaps() const {
+  bool have_rmaps;
+  std::map<string, int> type_rmap, name_rmap, rule_name_rmap;
+
+private:
+  void build_rmaps() {
     if (have_rmaps) return;
     build_rmap(type_map, type_rmap);
     build_rmap(name_map, name_rmap);
     build_rmap(rule_name_map, rule_name_rmap);
     have_rmaps = true;
   }
-  void build_rmap(const map<int, string> &f, std::map<string, int> &r) const {
+  void build_rmap(const map<int, string> &f, std::map<string, int> &r) {
     r.clear();
     for (std::map<int, string>::const_iterator p = f.begin(); p != f.end(); ++p)
       r[p->second] = p->first;
@@ -87,8 +88,6 @@ public:
       crush_destroy(crush);
   }
 
-  crush_map *get_crush_map() { return crush; }
-
   /* building */
   void create() {
     if (crush)
@@ -125,15 +124,12 @@ public:
 
   void set_tunables_legacy() {
     set_tunables_argonaut();
-    crush->straw_calc_version = 0;
   }
   void set_tunables_optimal() {
     set_tunables_firefly();
-    crush->straw_calc_version = 1;
   }
   void set_tunables_default() {
     set_tunables_bobtail();
-    crush->straw_calc_version = 1;
   }
 
   int get_choose_local_tries() const {
@@ -171,21 +167,13 @@ public:
     crush->chooseleaf_vary_r = n;
   }
 
-  int get_straw_calc_version() const {
-    return crush->straw_calc_version;
-  }
-  void set_straw_calc_version(int n) {
-    crush->straw_calc_version = n;
-  }
-
   bool has_argonaut_tunables() const {
     return
       crush->choose_local_tries == 2 &&
       crush->choose_local_fallback_tries == 5 &&
       crush->choose_total_tries == 19 &&
       crush->chooseleaf_descend_once == 0 &&
-      crush->chooseleaf_vary_r == 0 &&
-      crush->straw_calc_version == 0;
+      crush->chooseleaf_vary_r == 0;
   }
   bool has_bobtail_tunables() const {
     return
@@ -193,8 +181,7 @@ public:
       crush->choose_local_fallback_tries == 0 &&
       crush->choose_total_tries == 50 &&
       crush->chooseleaf_descend_once == 1 &&
-      crush->chooseleaf_vary_r == 0 &&
-      crush->straw_calc_version == 0;
+      crush->chooseleaf_vary_r == 0;
   }
   bool has_firefly_tunables() const {
     return
@@ -202,8 +189,7 @@ public:
       crush->choose_local_fallback_tries == 0 &&
       crush->choose_total_tries == 50 &&
       crush->chooseleaf_descend_once == 1 &&
-      crush->chooseleaf_vary_r == 1 &&
-      crush->straw_calc_version == 0;
+      crush->chooseleaf_vary_r == 1;
   }
 
   bool has_optimal_tunables() const {
@@ -230,14 +216,12 @@ public:
   bool has_v2_rules() const;
   bool has_v3_rules() const;
 
-  bool is_v2_rule(unsigned ruleid) const;
-  bool is_v3_rule(unsigned ruleid) const;
 
   // bucket types
   int get_num_type_names() const {
     return type_map.size();
   }
-  int get_type_id(const string& name) const {
+  int get_type_id(const string& name) {
     build_rmaps();
     if (type_rmap.count(name))
       return type_rmap[name];
@@ -256,14 +240,14 @@ public:
   }
 
   // item/bucket names
-  bool name_exists(const string& name) const {
+  bool name_exists(const string& name) {
     build_rmaps();
     return name_rmap.count(name);
   }
   bool item_exists(int i) {
     return name_map.count(i);
   }
-  int get_item_id(const string& name) const {
+  int get_item_id(const string& name) {
     build_rmaps();
     if (name_rmap.count(name))
       return name_rmap[name];
@@ -285,11 +269,11 @@ public:
   }
 
   // rule names
-  bool rule_exists(string name) const {
+  bool rule_exists(string name) {
     build_rmaps();
     return rule_name_rmap.count(name);
   }
-  int get_rule_id(string name) const {
+  int get_rule_id(string name) {
     build_rmaps();
     if (rule_name_rmap.count(name))
       return rule_name_rmap[name];
@@ -556,27 +540,19 @@ public:
    * @param id item id to check
    * @return weight of item
    */
-  int get_item_weight(int id) const;
-  float get_item_weightf(int id) const {
+  int get_item_weight(int id);
+  float get_item_weightf(int id) {
     return (float)get_item_weight(id) / (float)0x10000;
   }
-  int get_item_weight_in_loc(int id, const map<string,string> &loc);
-  float get_item_weightf_in_loc(int id, const map<string,string> &loc) {
-    return (float)get_item_weight_in_loc(id, loc) / (float)0x10000;
-  }
 
   int adjust_item_weight(CephContext *cct, int id, int weight);
   int adjust_item_weightf(CephContext *cct, int id, float weight) {
     return adjust_item_weight(cct, id, (int)(weight * (float)0x10000));
   }
-  int adjust_item_weight_in_loc(CephContext *cct, int id, int weight, const map<string,string>& loc);
-  int adjust_item_weightf_in_loc(CephContext *cct, int id, float weight, const map<string,string>& loc) {
-    return adjust_item_weight_in_loc(cct, id, (int)(weight * (float)0x10000), loc);
-  }
   void reweight(CephContext *cct);
 
   /// check if item id is present in the map hierarchy
-  bool check_item_present(int id) const;
+  bool check_item_present(int id);
 
 
   /*** devices ***/
@@ -665,7 +641,7 @@ public:
    * @param pmap [out] map of osd to weight
    * @return 0 for success, or negative error code
    */
-  int get_rule_weight_osd_map(unsigned ruleno, map<int,float> *pmap);
+  int get_rule_weight_map(unsigned ruleno, map<int,float> *pmap);
 
   /* modifiers */
   int add_rule(int len, int ruleset, int type, int minsize, int maxsize, int ruleno) {
@@ -767,6 +743,9 @@ private:
     crush_bucket *b = get_bucket(item);
     unsigned bucket_weight = b->weight;
 
+    // zero out the bucket weight
+    adjust_item_weight(cct, item, 0);
+
     // get where the bucket is located
     pair<string, string> bucket_location = get_immediate_parent(item);
 
@@ -777,12 +756,8 @@ private:
     crush_bucket *parent_bucket = get_bucket(parent_id);
 
     if (!IS_ERR(parent_bucket)) {
-      // zero out the bucket weight
-      crush_bucket_adjust_item_weight(crush, parent_bucket, item, 0);
-      adjust_item_weight(cct, parent_bucket->id, parent_bucket->weight);
-
       // remove the bucket from the parent
-      crush_bucket_remove_item(crush, parent_bucket, item);
+      crush_bucket_remove_item(parent_bucket, item);
     } else if (PTR_ERR(parent_bucket) != -ENOENT) {
       return PTR_ERR(parent_bucket);
     }
@@ -862,7 +837,7 @@ public:
 		 int *items, int *weights, int *idout) {
     if (type == 0)
       return -EINVAL;
-    crush_bucket *b = crush_make_bucket(crush, alg, hash, type, size, items, weights);
+    crush_bucket *b = crush_make_bucket(alg, hash, type, size, items, weights);
     assert(b);
     return crush_add_bucket(crush, bucketno, b, idout);
   }
@@ -901,11 +876,11 @@ public:
     return crush_find_rule(crush, ruleset, type, size);
   }
 
-  bool ruleset_exists(int ruleset) const {
+  bool ruleset_exists(int const ruleset) const {
     for (size_t i = 0; i < crush->max_rules; ++i) {
-      if (rule_exists(i) && crush->rules[i]->mask.ruleset == ruleset) {
-	return true;
-      }
+     if (crush->rules[i]->mask.ruleset == ruleset) {
+       return true;
+     }
     }
 
     return false;
