@@ -74,8 +74,6 @@ public:
     typedef map<snapid_t, old_inode_t> old_inodes_t;
     old_inodes_t old_inodes;
 
-    bufferlist _enc;
-
     fullbit(const fullbit& o);
     const fullbit& operator=(const fullbit& o);
 
@@ -84,26 +82,16 @@ public:
 	    const map<string,bufferptr> &xa, const string& sym,
 	    const bufferlist &sbl, __u8 st,
 	    const old_inodes_t *oi = NULL) :
-      //dn(d), dnfirst(df), dnlast(dl), dnv(v), 
-      //inode(i), dirfragtree(dft), xattrs(xa), symlink(sym), snapbl(sbl), dirty(dr) 
-      _enc(1024)
+      dn(d), dnfirst(df), dnlast(dl), dnv(v), inode(i), xattrs(xa), state(st)
     {
-      ::encode(d, _enc);
-      ::encode(df, _enc);
-      ::encode(dl, _enc);
-      ::encode(v, _enc);
-      ::encode(i, _enc);
-      ::encode(xa, _enc);
       if (i.is_symlink())
-	::encode(sym, _enc);
+	symlink = sym;
       if (i.is_dir()) {
-	::encode(dft, _enc);
-	::encode(sbl, _enc);
+	dirfragtree = dft;
+	snapbl = sbl;
       }
-      ::encode(st, _enc);
-      ::encode(oi ? true : false, _enc);
       if (oi)
-	::encode(*oi, _enc);
+	old_inodes = *oi;
     }
     fullbit(bufferlist::iterator &p) {
       decode(p);
@@ -153,19 +141,8 @@ public:
     unsigned char d_type;
     bool dirty;
 
-    bufferlist _enc;
-
     remotebit(const string& d, snapid_t df, snapid_t dl, version_t v, inodeno_t i, unsigned char dt, bool dr) : 
-      //dn(d), dnfirst(df), dnlast(dl), dnv(v), ino(i), d_type(dt), dirty(dr) { }
-      _enc(256) {
-      ::encode(d, _enc);
-      ::encode(df, _enc);
-      ::encode(dl, _enc);
-      ::encode(v, _enc);
-      ::encode(i, _enc);
-      ::encode(dt, _enc);
-      ::encode(dr, _enc);
-    }
+      dn(d), dnfirst(df), dnlast(dl), dnv(v), ino(i), d_type(dt), dirty(dr) { }
     remotebit(bufferlist::iterator &p) { decode(p); }
     remotebit(): dnfirst(0), dnlast(0), dnv(0), ino(0),
 	d_type('\0'), dirty(false) {}
@@ -191,17 +168,8 @@ public:
     version_t dnv;
     bool dirty;
 
-    bufferlist _enc;
-
     nullbit(const string& d, snapid_t df, snapid_t dl, version_t v, bool dr) : 
-      //dn(d), dnfirst(df), dnlast(dl), dnv(v), dirty(dr) { }
-      _enc(128) {
-      ::encode(d, _enc);
-      ::encode(df, _enc);
-      ::encode(dl, _enc);
-      ::encode(v, _enc);
-      ::encode(dr, _enc);
-    }
+      dn(d), dnfirst(df), dnlast(dl), dnv(v), dirty(dr) { }
     nullbit(bufferlist::iterator &p) { decode(p); }
     nullbit(): dnfirst(0), dnlast(0), dnv(0), dirty(false) {}
 
@@ -234,10 +202,10 @@ public:
 
   private:
     mutable bufferlist dnbl;
-    bool dn_decoded;
-    list<ceph::shared_ptr<fullbit> >   dfull;
-    list<remotebit> dremote;
-    list<nullbit>   dnull;
+    mutable bool dn_decoded;
+    mutable list<ceph::shared_ptr<fullbit> > dfull;
+    mutable list<remotebit> dremote;
+    mutable list<nullbit> dnull;
 
   public:
     dirlump() : state(0), nfull(0), nremote(0), nnull(0), dn_decoded(true) { }
@@ -253,9 +221,13 @@ public:
     bool is_dirty_dft() { return state & STATE_DIRTYDFT; }
     void mark_dirty_dft() { state |= STATE_DIRTYDFT; }
 
-    list<ceph::shared_ptr<fullbit> >   &get_dfull()   { return dfull; }
-    list<remotebit> &get_dremote() { return dremote; }
-    list<nullbit>   &get_dnull()   { return dnull; }
+    const list<ceph::shared_ptr<fullbit> > &get_dfull()   const { return dfull; }
+    const list<remotebit>                  &get_dremote() const { return dremote; }
+    const list<nullbit>                    &get_dnull()   const { return dnull; }
+
+    void add_dnull(nullbit const &n)                   { dnull.push_back(n); };
+    void add_dfull(ceph::shared_ptr<fullbit> const &p) { dfull.push_back(p); };
+    void add_dremote(remotebit const &r)               { dremote.push_back(r); };
 
     void print(dirfrag_t dirfrag, ostream& out) {
       out << "dirlump " << dirfrag << " v " << fnode.version
@@ -295,7 +267,7 @@ public:
       ::encode(dremote, dnbl);
       ::encode(dnull, dnbl);
     }
-    void _decode_bits() { 
+    void _decode_bits() const { 
       if (dn_decoded) return;
       bufferlist::iterator p = dnbl.begin();
       ::decode(dfull, p);
@@ -311,12 +283,11 @@ public:
   };
   WRITE_CLASS_ENCODER(dirlump)
 
-private:
   // my lumps.  preserve the order we added them in a list.
   list<dirfrag_t>         lump_order;
   map<dirfrag_t, dirlump> lump_map;
   list<ceph::shared_ptr<fullbit> > roots;
-
+public:
   list<pair<__u8,version_t> > table_tids;  // tableclient transactions
 
   inodeno_t opened_ino;
@@ -336,7 +307,9 @@ private:
   list<inodeno_t> truncate_start;        // start truncate 
   map<inodeno_t,uint64_t> truncate_finish;  // finished truncate (started in segment blah)
 
+public:
   vector<inodeno_t> destroyed_inodes;
+private:
 
   // idempotent op(s)
   list<pair<metareqid_t,uint64_t> > client_reqs;
@@ -344,6 +317,11 @@ private:
  public:
   void encode(bufferlist& bl) const;
   void decode(bufferlist::iterator& bl);
+  void get_inodes(std::set<inodeno_t> &inodes) const;
+  void get_paths(std::vector<std::string> &paths) const;
+  void get_dentries(std::map<dirfrag_t, std::set<std::string> > &dentries) const;
+  entity_name_t get_client_name() const {return client_name;}
+
   void dump(Formatter *f) const;
   static void generate_test_instances(list<EMetaBlob*>& ls);
   // soft stateadd
@@ -407,10 +385,10 @@ private:
   void add_null_dentry(dirlump& lump, CDentry *dn, bool dirty) {
     // add the dir
     lump.nnull++;
-    lump.get_dnull().push_back(nullbit(dn->get_name(), 
-				       dn->first, dn->last,
-				       dn->get_projected_version(), 
-				       dirty));
+    lump.add_dnull(nullbit(dn->get_name(), 
+			   dn->first, dn->last,
+			   dn->get_projected_version(), 
+			   dirty));
   }
 
   void add_remote_dentry(CDentry *dn, bool dirty) {
@@ -426,11 +404,11 @@ private:
       rdt = dn->get_projected_linkage()->get_remote_d_type();
     }
     lump.nremote++;
-    lump.get_dremote().push_back(remotebit(dn->get_name(), 
-					   dn->first, dn->last,
-					   dn->get_projected_version(), 
-					   rino, rdt,
-					   dirty));
+    lump.add_dremote(remotebit(dn->get_name(), 
+			       dn->first, dn->last,
+                               dn->get_projected_version(), 
+                               rino, rdt,
+                               dirty));
   }
 
   // return remote pointer to to-be-journaled inode
@@ -460,14 +438,14 @@ private:
       sr->encode(snapbl);
 
     lump.nfull++;
-    lump.get_dfull().push_back(ceph::shared_ptr<fullbit>(new fullbit(dn->get_name(), 
-									 dn->first, dn->last,
-									 dn->get_projected_version(), 
-									 *pi, in->dirfragtree,
-									 *in->get_projected_xattrs(),
-									 in->symlink, snapbl,
-									 state,
-									 &in->old_inodes)));
+    lump.add_dfull(ceph::shared_ptr<fullbit>(new fullbit(dn->get_name(), 
+                                                         dn->first, dn->last,
+                                                         dn->get_projected_version(), 
+                                                         *pi, in->dirfragtree,
+                                                         *in->get_projected_xattrs(),
+                                                         in->symlink, snapbl,
+                                                         state,
+                                                         &in->old_inodes)));
   }
 
   // convenience: primary or remote?  figure it out.
@@ -566,6 +544,13 @@ private:
   
   void add_dir_context(CDir *dir, int mode = TO_AUTH_SUBTREE_ROOT);
 
+  bool empty() {
+    return roots.empty() && lump_order.empty() && table_tids.empty() &&
+	   truncate_start.empty() && truncate_finish.empty() &&
+	   destroyed_inodes.empty() && client_reqs.empty() &&
+	   opened_ino == 0 && inotablev == 0 && sessionmapv == 0;
+  }
+
   void print(ostream& out) const {
     out << "[metablob";
     if (!lump_order.empty()) 
