@@ -637,7 +637,7 @@ inline bool operator>(const eversion_t& l, const eversion_t& r) {
 inline bool operator>=(const eversion_t& l, const eversion_t& r) {
   return (l.epoch == r.epoch) ? (l.version >= r.version):(l.epoch >= r.epoch);
 }
-inline ostream& operator<<(ostream& out, const eversion_t e) {
+inline ostream& operator<<(ostream& out, const eversion_t& e) {
   return out << e.epoch << "'" << e.version;
 }
 
@@ -654,11 +654,6 @@ struct objectstore_perf_stat_t {
   objectstore_perf_stat_t() :
     filestore_commit_latency(0), filestore_apply_latency(0) {}
 
-  bool operator==(const objectstore_perf_stat_t &r) const {
-    return filestore_commit_latency == r.filestore_commit_latency &&
-      filestore_apply_latency == r.filestore_apply_latency;
-  }
-
   void add(const objectstore_perf_stat_t &o) {
     filestore_commit_latency += o.filestore_commit_latency;
     filestore_apply_latency += o.filestore_apply_latency;
@@ -722,9 +717,7 @@ inline bool operator==(const osd_stat_t& l, const osd_stat_t& r) {
     l.snap_trim_queue_len == r.snap_trim_queue_len &&
     l.num_snap_trimming == r.num_snap_trimming &&
     l.hb_in == r.hb_in &&
-    l.hb_out == r.hb_out &&
-    l.op_queue_age_hist == r.op_queue_age_hist &&
-    l.fs_perf_stat == r.fs_perf_stat;
+    l.hb_out == r.hb_out;
 }
 inline bool operator!=(const osd_stat_t& l, const osd_stat_t& r) {
   return !(l == r);
@@ -818,10 +811,9 @@ struct pg_pool_t {
   }
 
   enum {
-    FLAG_HASHPSPOOL = 1<<0, // hash pg seed and pool together (instead of adding)
-    FLAG_FULL       = 1<<1, // pool is full
+    FLAG_HASHPSPOOL = 1, // hash pg seed and pool together (instead of adding)
+    FLAG_FULL       = 2, // pool is full
     FLAG_DEBUG_FAKE_EC_POOL = 1<<2, // require ReplicatedPG to act like an EC pg
-    FLAG_INCOMPLETE_CLONES = 1<<3, // may have incomplete clones (bc we are/were an overlay)
   };
 
   static const char *get_flag_name(int f) {
@@ -829,7 +821,6 @@ struct pg_pool_t {
     case FLAG_HASHPSPOOL: return "hashpspool";
     case FLAG_FULL: return "full";
     case FLAG_DEBUG_FAKE_EC_POOL: return "require_local_rollback";
-    case FLAG_INCOMPLETE_CLONES: return "incomplete_clones";
     default: return "???";
     }
   }
@@ -877,18 +868,6 @@ struct pg_pool_t {
   const char *get_cache_mode_name() const {
     return get_cache_mode_name(cache_mode);
   }
-  bool cache_mode_requires_hit_set() const {
-    switch (cache_mode) {
-    case CACHEMODE_NONE:
-    case CACHEMODE_FORWARD:
-    case CACHEMODE_READONLY:
-      return false;
-    case CACHEMODE_WRITEBACK:
-      return true;
-    default:
-      assert(0 == "implement me");
-    }
-  }
 
   uint64_t flags;           ///< FLAG_*
   __u8 type;                ///< TYPE_*
@@ -937,29 +916,11 @@ public:
 
   bool is_tier() const { return tier_of >= 0; }
   bool has_tiers() const { return !tiers.empty(); }
-  void clear_tier() {
-    tier_of = -1;
-    clear_read_tier();
-    clear_write_tier();
-    clear_tier_tunables();
-  }
+  void clear_tier() { tier_of = -1; }
   bool has_read_tier() const { return read_tier >= 0; }
   void clear_read_tier() { read_tier = -1; }
   bool has_write_tier() const { return write_tier >= 0; }
   void clear_write_tier() { write_tier = -1; }
-  void clear_tier_tunables() {
-    if (cache_mode != CACHEMODE_NONE)
-      flags |= FLAG_INCOMPLETE_CLONES;
-    cache_mode = CACHEMODE_NONE;
-
-    target_max_bytes = 0;
-    target_max_objects = 0;
-    cache_target_dirty_ratio_micro = 0;
-    cache_target_full_ratio_micro = 0;
-    hit_set_params = HitSet::Params();
-    hit_set_period = 0;
-    hit_set_count = 0;
-  }
 
   uint64_t target_max_bytes;   ///< tiering: target max pool size
   uint64_t target_max_objects; ///< tiering: target max pool size
@@ -1003,7 +964,6 @@ public:
   void dump(Formatter *f) const;
 
   uint64_t get_flags() const { return flags; }
-  bool has_flag(uint64_t f) const { return flags & f; }
 
   /// This method will later return true for ec pools as well
   bool ec_pool() const {
@@ -1013,11 +973,6 @@ public:
     return ec_pool() || flags & FLAG_DEBUG_FAKE_EC_POOL;
   }
 
-  /// true if incomplete clones may be present
-  bool allow_incomplete_clones() const {
-    return cache_mode != CACHEMODE_NONE || has_flag(FLAG_INCOMPLETE_CLONES);
-  }
-
   unsigned get_type() const { return type; }
   unsigned get_size() const { return size; }
   unsigned get_min_size() const { return min_size; }
@@ -1752,13 +1707,30 @@ struct pg_interval_t {
   void dump(Formatter *f) const;
   static void generate_test_instances(list<pg_interval_t*>& o);
 
+  /**
+   * Determines whether there is an interval change
+   */
+  static bool is_new_interval(
+    int old_acting_primary,                     ///< [in] primary as of lastmap
+    int new_acting_primary,                     ///< [in] primary as of lastmap
+    const vector<int> &old_acting,              ///< [in] acting as of lastmap
+    const vector<int> &new_acting,              ///< [in] acting as of osdmap
+    int old_up_primary,                         ///< [in] up primary of lastmap
+    int new_up_primary,                         ///< [in] up primary of osdmap
+    const vector<int> &old_up,                  ///< [in] up as of lastmap
+    const vector<int> &new_up,                  ///< [in] up as of osdmap
+    ceph::shared_ptr<const OSDMap> osdmap,  ///< [in] current map
+    ceph::shared_ptr<const OSDMap> lastmap, ///< [in] last map
+    pg_t pgid                                   ///< [in] pgid for pg
+    );
+
   /**
    * Integrates a new map into *past_intervals, returns true
    * if an interval was closed out.
    */
   static bool check_new_interval(
     int old_acting_primary,                     ///< [in] primary as of lastmap
-    int new_acting_primary,                     ///< [in] primary as of lastmap
+    int new_acting_primary,                     ///< [in] primary as of osdmap
     const vector<int> &old_acting,              ///< [in] acting as of lastmap
     const vector<int> &new_acting,              ///< [in] acting as of osdmap
     int old_up_primary,                         ///< [in] up primary of lastmap
@@ -1769,7 +1741,6 @@ struct pg_interval_t {
     epoch_t last_epoch_clean,                   ///< [in] current
     ceph::shared_ptr<const OSDMap> osdmap,  ///< [in] current map
     ceph::shared_ptr<const OSDMap> lastmap, ///< [in] last map
-    int64_t poolid,                             ///< [in] pool for pg
     pg_t pgid,                                  ///< [in] pgid for pg
     map<epoch_t, pg_interval_t> *past_intervals,///< [out] intervals
     ostream *out = 0                            ///< [out] debug ostream
@@ -1856,7 +1827,7 @@ inline ostream& operator<<(ostream& out, const pg_query_t& q) {
 class PGBackend;
 class ObjectModDesc {
   bool can_local_rollback;
-  bool rollback_info_completed;
+  bool stashed;
 public:
   class Visitor {
   public:
@@ -1876,22 +1847,22 @@ public:
     CREATE = 4,
     UPDATE_SNAPS = 5
   };
-  ObjectModDesc() : can_local_rollback(true), rollback_info_completed(false) {}
+  ObjectModDesc() : can_local_rollback(true), stashed(false) {}
   void claim(ObjectModDesc &other) {
     bl.clear();
     bl.claim(other.bl);
     can_local_rollback = other.can_local_rollback;
-    rollback_info_completed = other.rollback_info_completed;
+    stashed = other.stashed;
   }
   void claim_append(ObjectModDesc &other) {
-    if (!can_local_rollback || rollback_info_completed)
+    if (!can_local_rollback || stashed)
       return;
     if (!other.can_local_rollback) {
       mark_unrollbackable();
       return;
     }
     bl.claim_append(other.bl);
-    rollback_info_completed = other.rollback_info_completed;
+    stashed = other.stashed;
   }
   void swap(ObjectModDesc &other) {
     bl.swap(other.bl);
@@ -1900,16 +1871,16 @@ public:
     other.can_local_rollback = can_local_rollback;
     can_local_rollback = temp;
 
-    temp = other.rollback_info_completed;
-    other.rollback_info_completed = rollback_info_completed;
-    rollback_info_completed = temp;
+    temp = other.stashed;
+    other.stashed = stashed;
+    stashed = temp;
   }
   void append_id(ModID id) {
     uint8_t _id(id);
     ::encode(_id, bl);
   }
   void append(uint64_t old_size) {
-    if (!can_local_rollback || rollback_info_completed)
+    if (!can_local_rollback || stashed)
       return;
     ENCODE_START(1, 1, bl);
     append_id(APPEND);
@@ -1917,7 +1888,7 @@ public:
     ENCODE_FINISH(bl);
   }
   void setattrs(map<string, boost::optional<bufferlist> > &old_attrs) {
-    if (!can_local_rollback || rollback_info_completed)
+    if (!can_local_rollback || stashed)
       return;
     ENCODE_START(1, 1, bl);
     append_id(SETATTRS);
@@ -1925,25 +1896,24 @@ public:
     ENCODE_FINISH(bl);
   }
   bool rmobject(version_t deletion_version) {
-    if (!can_local_rollback || rollback_info_completed)
+    if (!can_local_rollback || stashed)
       return false;
     ENCODE_START(1, 1, bl);
     append_id(DELETE);
     ::encode(deletion_version, bl);
     ENCODE_FINISH(bl);
-    rollback_info_completed = true;
+    stashed = true;
     return true;
   }
   void create() {
-    if (!can_local_rollback || rollback_info_completed)
+    if (!can_local_rollback || stashed)
       return;
-    rollback_info_completed = true;
     ENCODE_START(1, 1, bl);
     append_id(CREATE);
     ENCODE_FINISH(bl);
   }
   void update_snaps(set<snapid_t> &old_snaps) {
-    if (!can_local_rollback || rollback_info_completed)
+    if (!can_local_rollback || stashed)
       return;
     ENCODE_START(1, 1, bl);
     append_id(UPDATE_SNAPS);
@@ -2107,10 +2077,6 @@ struct pg_log_t {
   // We can rollback rollback-able entries > can_rollback_to
   eversion_t can_rollback_to;
 
-  // always <= can_rollback_to, indicates how far stashed rollback
-  // data can be found
-  eversion_t rollback_info_trimmed_to;
-
   list<pg_log_entry_t> log;  // the actual log.
   
   pg_log_t() {}
@@ -2542,29 +2508,6 @@ struct SnapSet {
   void decode(bufferlist::iterator& bl);
   void dump(Formatter *f) const;
   static void generate_test_instances(list<SnapSet*>& o);  
-
-  SnapContext get_ssc_as_of(snapid_t as_of) const {
-    SnapContext out;
-    out.seq = as_of;
-    for (vector<snapid_t>::const_iterator i = snaps.begin();
-	 i != snaps.end();
-	 ++i) {
-      if (*i <= as_of)
-	out.snaps.push_back(*i);
-    }
-    return out;
-  }
-
-  // return min element of snaps > after, return max if no such element
-  snapid_t get_first_snap_after(snapid_t after, snapid_t max) const {
-    for (vector<snapid_t>::const_reverse_iterator i = snaps.rbegin();
-	 i != snaps.rend();
-	 ++i) {
-      if (*i > after)
-	return *i;
-    }
-    return max;
-  }
 };
 WRITE_CLASS_ENCODER(SnapSet)
 
@@ -2835,21 +2778,19 @@ public:
       }
     }
 
-    bool get_write(OpRequestRef op, bool greedy=false) {
-      if (get_write_lock(greedy)) {
+    bool get_write(OpRequestRef op) {
+      if (get_write_lock()) {
 	return true;
       } // else
       if (op)
 	waiters.push_back(op);
       return false;
     }
-    bool get_write_lock(bool greedy=false) {
-      if (!greedy) {
-	// don't starve anybody!
-	if (!waiters.empty() ||
-	    backfill_read_marker) {
-	  return false;
-	}
+    bool get_write_lock() {
+      // don't starve anybody!
+      if (!waiters.empty() ||
+	  backfill_read_marker) {
+	return false;
       }
       switch (state) {
       case RWNONE:
@@ -2898,10 +2839,7 @@ public:
     return rwstate.get_read(op);
   }
   bool get_write(OpRequestRef op) {
-    return rwstate.get_write(op, false);
-  }
-  bool get_write_greedy(OpRequestRef op) {
-    return rwstate.get_write(op, true);
+    return rwstate.get_write(op);
   }
   bool get_snaptrimmer_write() {
     if (rwstate.get_write_lock()) {
