@@ -37,12 +37,14 @@ void Journaler::set_writeable()
   readonly = false;
 }
 
-void Journaler::create(ceph_file_layout *l)
+void Journaler::create(ceph_file_layout *l, stream_format_t const sf)
 {
   assert(!readonly);
   ldout(cct, 1) << "create blank journal" << dendl;
   state = STATE_ACTIVE;
 
+  stream_format = sf;
+  journal_stream.set_format(sf);
   set_layout(l);
 
   prezeroing_pos = prezero_pos = write_pos = flush_pos = safe_pos =
@@ -181,8 +183,7 @@ void Journaler::_finish_reread_head(int r, bufferlist& bl, Context *finish)
   trimmed_pos = trimming_pos = h.trimmed_pos;
   init_headers(h);
   state = STATE_ACTIVE;
-  finish->finish(r);
-  delete finish;
+  finish->complete(r);
 }
 
 void Journaler::_finish_read_head(int r, bufferlist& bl)
@@ -227,6 +228,8 @@ void Journaler::_finish_read_head(int r, bufferlist& bl)
 
   init_headers(h);
   set_layout(&h.layout);
+  stream_format = h.stream_format;
+  journal_stream.set_format(h.stream_format);
 
   ldout(cct, 1) << "_finish_read_head " << h << ".  probing for end of log (from " << write_pos << ")..." << dendl;
   C_ProbeEnd *fin = new C_ProbeEnd(this);
@@ -261,8 +264,7 @@ void Journaler::_finish_reprobe(int r, uint64_t new_end, Context *onfinish) {
 	  << dendl;
   prezeroing_pos = prezero_pos = write_pos = flush_pos = safe_pos = new_end;
   state = STATE_ACTIVE;
-  onfinish->finish(r);
-  delete onfinish;
+  onfinish->complete(r);
 }
 
 void Journaler::_finish_probe_end(int r, uint64_t end)
@@ -341,6 +343,7 @@ void Journaler::write_head(Context *oncommit)
   last_written.expire_pos = expire_pos;
   last_written.unused_field = expire_pos;
   last_written.write_pos = safe_pos;
+  last_written.stream_format = stream_format;
   ldout(cct, 10) << "write_head " << last_written << dendl;
   
   last_wrote_head = ceph_clock_now(cct);
@@ -367,8 +370,7 @@ void Journaler::_finish_write_head(int r, Header &wrote, Context *oncommit)
   ldout(cct, 10) << "_finish_write_head " << wrote << dendl;
   last_committed = wrote;
   if (oncommit) {
-    oncommit->finish(r);
-    delete oncommit;
+    oncommit->complete(r);
   }
 
   trim();  // trim?
@@ -432,6 +434,7 @@ void Journaler::_finish_flush(int r, uint64_t start, utime_t stamp)
 }
 
 
+
 uint64_t Journaler::append_entry(bufferlist& bl)
 {
   assert(!readonly);
@@ -460,12 +463,11 @@ uint64_t Journaler::append_entry(bufferlist& bl)
     }
   }
 	
-  ldout(cct, 10) << "append_entry len " << bl.length() << " to " << write_pos << "~" << (bl.length() + sizeof(uint32_t)) << dendl;
   
   // append
-  ::encode(s, write_buf);
-  write_buf.claim_append(bl);
-  write_pos += sizeof(s) + s;
+  size_t wrote = journal_stream.write(bl, &write_buf, write_pos);
+  ldout(cct, 10) << "append_entry len " << s << " to " << write_pos << "~" << wrote << dendl;
+  write_pos += wrote;
 
   // flush previous object?
   uint64_t su = get_layout_period();
@@ -563,8 +565,7 @@ void Journaler::wait_for_flush(Context *onsafe)
     ldout(cct, 10) << "flush nothing to flush, (prezeroing/prezero)/write/flush/safe pointers at " 
 	     << "(" << prezeroing_pos << "/" << prezero_pos << ")/" << write_pos << "/" << flush_pos << "/" << safe_pos << dendl;
     if (onsafe) {
-      onsafe->finish(0);
-      delete onsafe;
+      onsafe->complete(0);
       onsafe = 0;
     }
     return;
@@ -584,8 +585,7 @@ void Journaler::flush(Context *onsafe)
     ldout(cct, 10) << "flush nothing to flush, (prezeroing/prezero)/write/flush/safe pointers at "
 	     << "(" << prezeroing_pos << "/" << prezero_pos << ")/" << write_pos << "/" << flush_pos << "/" << safe_pos << dendl;
     if (onsafe) {
-      onsafe->finish(0);
-      delete onsafe;
+      onsafe->complete(0);
     }
   } else {
     if (1) {
@@ -731,8 +731,7 @@ void Journaler::_finish_read(int r, uint64_t offset, bufferlist& bl)
     if (on_readable) {
       Context *f = on_readable;
       on_readable = 0;
-      f->finish(r);
-      delete f;
+      f->complete(r);
     }
     return;
   }
@@ -779,8 +778,7 @@ void Journaler::_assimilate_prefetch()
     if (on_readable) {
       Context *f = on_readable;
       on_readable = 0;
-      f->finish(0);
-      delete f;
+      f->complete(0);
     }
   }
 }
@@ -865,6 +863,7 @@ void Journaler::_prefetch()
   }
 }
 
+
 /*
  * _is_readable() - return true if next entry is ready.
  */
@@ -874,20 +873,14 @@ bool Journaler::_is_readable()
   if (read_pos == write_pos)
     return false;
 
-  // have enough for entry size?
-  uint32_t s = 0;
-  bufferlist::iterator p = read_buf.begin();
-  if (read_buf.length() >= sizeof(s))
-    ::decode(s, p);
-
-  // entry and payload?
-  if (read_buf.length() >= sizeof(s) &&
-      read_buf.length() >= sizeof(s) + s) 
-    return true;  // yep, next entry is ready.
+  // Check if the retrieve bytestream has enough for an entry
+  uint64_t need;
+  if (journal_stream.readable(read_buf, &need)) {
+    return true;
+  }
 
   ldout (cct, 10) << "_is_readable read_buf.length() == " << read_buf.length()
-		  << ", but need " << s + sizeof(s)
-		  << " for next entry; fetch_len is " << fetch_len << dendl;
+		  << ", but need " << need << " for next entry; fetch_len is " << fetch_len << dendl;
 
   // partial fragment at the end?
   if (received_pos == write_pos) {
@@ -906,11 +899,9 @@ bool Journaler::_is_readable()
     return false;
   }
 
-  uint64_t need = sizeof(s) + s;
   if (need > fetch_len) {
-    temp_fetch_len = sizeof(s) + s;
-    ldout(cct, 10) << "_is_readable noting temp_fetch_len " << temp_fetch_len
-	     << " for len " << s << " entry" << dendl;
+    temp_fetch_len = need;
+    ldout(cct, 10) << "_is_readable noting temp_fetch_len " << temp_fetch_len << dendl;
   }
 
   ldout(cct, 10) << "_is_readable: not readable, returning false" << dendl;
@@ -927,6 +918,44 @@ bool Journaler::is_readable()
   return r;
 }
 
+class Journaler::C_EraseFinish : public Context {
+  Journaler *journaler;
+  Context *completion;
+  public:
+  C_EraseFinish(Journaler *j, Context *c) : journaler(j), completion(c) {}
+  void finish(int r) {
+    journaler->_erase_finish(r, completion);
+  }
+};
+
+/**
+ * Entirely erase the journal, including header.  For use when you
+ * have already made a copy of the journal somewhere else.
+ */
+void Journaler::erase(Context *completion)
+{
+  // Async delete the journal data
+  uint64_t first = trimmed_pos / get_layout_period();
+  uint64_t num = (write_pos - trimmed_pos) / get_layout_period() + 2;
+  filer.purge_range(ino, &layout, SnapContext(), first, num, ceph_clock_now(cct), 0,
+      new C_EraseFinish(this, completion));
+
+  // We will not start the operation to delete the header until _erase_finish has
+  // seen the data deletion succeed: otherwise if there was an error deleting data
+  // we might prematurely delete the header thereby lose our reference to the data.
+}
+
+void Journaler::_erase_finish(int data_result, Context *completion)
+{
+  if (data_result == 0) {
+    // Async delete the journal header
+    filer.purge_range(ino, &layout, SnapContext(), 0, 1, ceph_clock_now(cct), 0,
+        completion);
+  } else {
+    lderr(cct) << "Failed to delete journal " << ino << " data: " << cpp_strerror(data_result) << dendl;
+    completion->complete(data_result);
+  }
+}
 
 /* try_read_entry(bl)
  *  read entry into bl if it's ready.
@@ -938,28 +967,17 @@ bool Journaler::try_read_entry(bufferlist& bl)
     ldout(cct, 10) << "try_read_entry at " << read_pos << " not readable" << dendl;
     return false;
   }
-  
-  uint32_t s;
-  {
-    bufferlist::iterator p = read_buf.begin();
-    ::decode(s, p);
-  }
-  assert(read_buf.length() >= sizeof(s) + s);
-  
-  ldout(cct, 10) << "try_read_entry at " << read_pos << " reading " 
-	   << read_pos << "~" << (sizeof(s)+s) << " (have " << read_buf.length() << ")" << dendl;
 
-  if (s == 0) {
-    ldout(cct, 0) << "try_read_entry got 0 len entry at offset " << read_pos << dendl;
-    error = -EINVAL;
-    return false;
+  uint64_t start_ptr;
+  size_t consumed = journal_stream.read(read_buf, &bl, &start_ptr);
+  if (stream_format >= JOURNAL_FORMAT_RESILIENT) {
+    assert(start_ptr == read_pos);
   }
 
-  // do it
-  assert(bl.length() == 0);
-  read_buf.splice(0, sizeof(s));
-  read_buf.splice(0, s, &bl);
-  read_pos += sizeof(s) + s;
+  ldout(cct, 10) << "try_read_entry at " << read_pos << " read " 
+	   << read_pos << "~" << consumed << " (have " << read_buf.length() << ")" << dendl;
+
+  read_pos += consumed;
 
   // prefetch?
   _prefetch();
@@ -1060,8 +1078,7 @@ void Journaler::handle_write_error(int r)
 {
   lderr(cct) << "handle_write_error " << cpp_strerror(r) << dendl;
   if (on_write_error) {
-    on_write_error->finish(r);
-    delete on_write_error;
+    on_write_error->complete(r);
     on_write_error = NULL;
   } else {
     assert(0 == "unhandled write error");
@@ -1069,4 +1086,128 @@ void Journaler::handle_write_error(int r)
 }
 
 
-// eof.
+/**
+ * Test whether the 'read_buf' byte stream has enough data to read
+ * an entry
+ *
+ * sets 'next_envelope_size' to the number of bytes needed to advance (enough
+ * to get the next header if header was unavailable, or enough to get the whole
+ * next entry if the header was available but the body wasn't).
+ */
+bool JournalStream::readable(bufferlist &read_buf, uint64_t *need) const
+{
+  assert(need != NULL);
+
+  uint32_t entry_size = 0;
+  uint64_t entry_sentinel = 0;
+  bufferlist::iterator p = read_buf.begin();
+
+  // Do we have enough data to decode an entry prefix?
+  if (format >= JOURNAL_FORMAT_RESILIENT) {
+    *need = sizeof(entry_size) + sizeof(entry_sentinel);
+  } else {
+    *need = sizeof(entry_size);
+  }
+  if (read_buf.length() >= *need) {
+    if (format >= JOURNAL_FORMAT_RESILIENT) {
+      ::decode(entry_sentinel, p);
+      if (entry_sentinel != sentinel) {
+        throw buffer::malformed_input("Invalid sentinel"); 
+      }
+    }
+
+    ::decode(entry_size, p);
+  } else {
+    return false;
+  }
+
+  // Do we have enough data to decode an entry prefix, payload and suffix?
+  if (format >= JOURNAL_FORMAT_RESILIENT) {
+    *need = JOURNAL_ENVELOPE_RESILIENT + entry_size;
+  } else {
+    *need = JOURNAL_ENVELOPE_LEGACY + entry_size;
+  }
+  if (read_buf.length() >= *need) {
+    return true;  // No more bytes needed
+  }
+
+  return false;
+}
+
+
+/**
+ * Consume one entry from a journal byte stream 'from', splicing a
+ * serialized LogEvent blob into 'entry'.
+ *
+ * 'entry' must be non null and point to an empty bufferlist.
+ *
+ * 'from' must contain sufficient valid data (i.e. readable is true).
+ *
+ * 'start_ptr' will be set to the entry's start pointer, if the collection
+ * format provides it.  It may not be null.
+ *
+ * @returns The number of bytes consumed from the `from` byte stream.  Note
+ *          that this is not equal to the length of `entry`, which contains
+ *          the inner serialized LogEvent and not the envelope.
+ */
+size_t JournalStream::read(bufferlist &from, bufferlist *entry, uint64_t *start_ptr)
+{
+  assert(start_ptr != NULL);
+  assert(entry != NULL);
+  assert(entry->length() == 0);
+
+  uint32_t entry_size = 0;
+
+  // Consume envelope prefix: entry_size and entry_sentinel
+  bufferlist::iterator from_ptr = from.begin();
+  if (format >= JOURNAL_FORMAT_RESILIENT) {
+    uint64_t entry_sentinel = 0;
+    ::decode(entry_sentinel, from_ptr);
+    // Assertion instead of clean check because of precondition of this
+    // fn is that readable() already passed
+    assert(entry_sentinel == sentinel);
+  }
+  ::decode(entry_size, from_ptr);
+  assert(entry_size != 0);
+
+  // Read out the payload
+  from_ptr.copy(entry_size, *entry);
+
+  // Consume the envelope suffix (start_ptr)
+  if (format >= JOURNAL_FORMAT_RESILIENT) {
+    ::decode(*start_ptr, from_ptr);
+  } else {
+    *start_ptr = 0;
+  }
+
+  // Trim the input buffer to discard the bytes we have consumed
+  from.splice(0, from_ptr.get_off());
+
+  return from_ptr.get_off();
+}
+
+
+/**
+ * Append one entry
+ */
+size_t JournalStream::write(bufferlist &entry, bufferlist *to, uint64_t const &start_ptr)
+{
+  assert(to != NULL);
+
+  uint32_t const entry_size = entry.length();
+  if (format >= JOURNAL_FORMAT_RESILIENT) {
+    ::encode(sentinel, *to);
+  }
+  ::encode(entry_size, *to);
+  to->claim_append(entry);
+  if (format >= JOURNAL_FORMAT_RESILIENT) {
+    ::encode(start_ptr, *to);
+  }
+
+  if (format >= JOURNAL_FORMAT_RESILIENT) {
+    return JOURNAL_ENVELOPE_RESILIENT + entry_size;
+  } else {
+    return JOURNAL_ENVELOPE_LEGACY + entry_size;
+  }
+}
+
