@@ -144,7 +144,6 @@ class RadosThread(threading.Thread):
 POLL_TIME_INCR = 0.5
 
 def run_in_thread(target, args, timeout=0):
-    import sys
     interrupt = False
 
     countdown = timeout
@@ -182,7 +181,7 @@ def run_in_thread(target, args, timeout=0):
 class Rados(object):
     """librados python wrapper"""
     def require_state(self, *args):
-        """ 
+        """
         Checks if the Rados object is in a special state
 
         :raises: RadosStateError
@@ -246,8 +245,7 @@ Rados object in state %s." % (self.state))
 
     def shutdown(self):
         """
-        Disconnects from the cluster.  Call this explicitly when a
-        Rados.connect()ed object is no longer used.
+        Disconnects from the cluster.
         """
         if (self.__dict__.has_key("state") and self.state != "shutdown"):
             run_in_thread(self.librados.rados_shutdown, (self.cluster,))
@@ -261,10 +259,13 @@ Rados object in state %s." % (self.state))
         self.shutdown()
         return False
 
+    def __del__(self):
+        self.shutdown()
+
     def version(self):
         """
         Get the version number of the ``librados`` C library.
-    
+
         :returns: a tuple of ``(major, minor, extra)`` components of the
                   librados version
         """
@@ -408,7 +409,7 @@ Rados object in state %s." % (self.state))
 
     def connect(self, timeout=0):
         """
-        Connect to the cluster.  Use shutdown() to release resources.
+        Connect to the cluster.
         """
         self.require_state("configuring")
         ret = run_in_thread(self.librados.rados_connect, (self.cluster,),
@@ -420,14 +421,14 @@ Rados object in state %s." % (self.state))
     def get_cluster_stats(self):
         """
         Read usage info about the cluster
-        
+
         This tells you total space, space used, space available, and number
         of objects. These are not updated immediately when data is written,
         they are eventually consistent.
 
         :returns: dict - contains the following keys:
 
-            - ``kb`` (int) - total space 
+            - ``kb`` (int) - total space
 
             - ``kb_used`` (int) - space used
 
@@ -519,7 +520,7 @@ Rados object in state %s." % (self.state))
         The pool is removed from the cluster immediately,
         but the actual data is deleted in the background.
 
-        :param pool_name: name of the pool to delete 
+        :param pool_name: name of the pool to delete
         :type pool_name: str
 
         :raises: :class:`TypeError`, :class:`Error`
@@ -534,7 +535,7 @@ Rados object in state %s." % (self.state))
 
     def list_pools(self):
         """
-        Gets a list of pool names. 
+        Gets a list of pool names.
 
         :returns: list - of pool names.
         """
@@ -571,13 +572,13 @@ Rados object in state %s." % (self.state))
         Create an io context
 
         The io context allows you to perform operations within a particular
-        pool. 
+        pool.
 
-        :param ioctx_name: name of the pool 
+        :param ioctx_name: name of the pool
         :type ioctx_name: str
 
         :raises: :class:`TypeError`, :class:`Error`
-        :returns: Ioctx - Rados Ioctx object 
+        :returns: Ioctx - Rados Ioctx object
         """
         self.require_state("connected")
         if not isinstance(ioctx_name, str):
@@ -594,7 +595,6 @@ Rados object in state %s." % (self.state))
         mon_command[_target](cmd, inbuf, outbuf, outbuflen, outs, outslen)
         returns (int ret, string outbuf, string outs)
         """
-        import sys
         self.require_state("connected")
         outbufp = pointer(pointer(c_char()))
         outbuflen = c_long()
@@ -632,7 +632,6 @@ Rados object in state %s." % (self.state))
         osd_command(osdid, cmd, inbuf, outbuf, outbuflen, outs, outslen)
         returns (int ret, string outbuf, string outs)
         """
-        import sys
         self.require_state("connected")
         outbufp = pointer(pointer(c_char()))
         outbuflen = c_long()
@@ -662,7 +661,6 @@ Rados object in state %s." % (self.state))
         pg_command(pgid, cmd, inbuf, outbuf, outbuflen, outs, outslen)
         returns (int ret, string outbuf, string outs)
         """
-        import sys
         self.require_state("connected")
         outbufp = pointer(pointer(c_char()))
         outbuflen = c_long()
@@ -780,7 +778,7 @@ ioctx '%s'" % self.ioctx.name)
         """
         Get the next Snapshot
 
-        :raises: :class:`Error`, StopIteration 
+        :raises: :class:`Error`, StopIteration
         :returns: Snap - next snapshot
         """
         if (self.cur_snap >= self.max_snap):
@@ -829,14 +827,11 @@ class Snap(object):
 
 class Completion(object):
     """completion object"""
-    def __init__(self, ioctx, rados_comp, oncomplete, onsafe,
-                 complete_cb, safe_cb):
+    def __init__(self, ioctx, rados_comp, oncomplete, onsafe):
         self.rados_comp = rados_comp
         self.oncomplete = oncomplete
         self.onsafe = onsafe
         self.ioctx = ioctx
-        self.complete_cb = complete_cb
-        self.safe_cb = safe_cb
 
     def wait_for_safe(self):
         """
@@ -873,8 +868,8 @@ class Completion(object):
                              (self.rados_comp,))
 
     def __del__(self):
-        """ 
-        Release a completion 
+        """
+        Release a completion
 
         Call this when you no longer need the completion. It may not be
         freed immediately if the operation is not acked and committed.
@@ -882,8 +877,6 @@ class Completion(object):
         run_in_thread(self.ioctx.librados.rados_aio_release,
                       (self.rados_comp,))
 
-RADOS_CB = CFUNCTYPE(c_int, c_void_p, c_void_p)
-
 class Ioctx(object):
     """rados.Ioctx object"""
     def __init__(self, name, librados, io):
@@ -894,6 +887,9 @@ class Ioctx(object):
         self.locator_key = ""
         self.safe_cbs = {}
         self.complete_cbs = {}
+        RADOS_CB = CFUNCTYPE(c_int, c_void_p, c_void_p)
+        self.__aio_safe_cb_c = RADOS_CB(self.__aio_safe_cb)
+        self.__aio_complete_cb_c = RADOS_CB(self.__aio_complete_cb)
         self.lock = threading.Lock()
 
     def __enter__(self):
@@ -907,8 +903,8 @@ class Ioctx(object):
         self.close()
 
     def __aio_safe_cb(self, completion, _):
-        """ 
-        Callback to onsafe() for asynchronous operations 
+        """
+        Callback to onsafe() for asynchronous operations
         """
         cb = None
         with self.lock:
@@ -918,8 +914,8 @@ class Ioctx(object):
         return 0
 
     def __aio_complete_cb(self, completion, _):
-        """ 
-        Callback to oncomplete() for asynchronous operations 
+        """
+        Callback to oncomplete() for asynchronous operations
         """
         cb = None
         with self.lock:
@@ -940,23 +936,22 @@ class Ioctx(object):
         :type onsafe: completion
 
         :raises: :class:`Error`
-        :returns: completion object 
+        :returns: completion object
         """
         completion = c_void_p(0)
         complete_cb = None
         safe_cb = None
         if oncomplete:
-            complete_cb = RADOS_CB(self.__aio_complete_cb)
+            complete_cb = self.__aio_complete_cb_c
         if onsafe:
-            safe_cb = RADOS_CB(self.__aio_safe_cb)
+            safe_cb = self.__aio_safe_cb_c
         ret = run_in_thread(self.librados.rados_aio_create_completion,
                             (c_void_p(0), complete_cb, safe_cb,
                             byref(completion)))
         if ret < 0:
             raise make_ex(ret, "error getting a completion")
         with self.lock:
-            completion_obj = Completion(self, completion, oncomplete, onsafe,
-                                        complete_cb, safe_cb)
+            completion_obj = Completion(self, completion, oncomplete, onsafe)
             if oncomplete:
                 self.complete_cbs[completion.value] = completion_obj
             if onsafe:
@@ -984,7 +979,7 @@ class Ioctx(object):
         :type onsafe: completion
 
         :raises: :class:`Error`
-        :returns: completion object 
+        :returns: completion object
         """
         completion = self.__get_completion(oncomplete, onsafe)
         ret = run_in_thread(self.librados.rados_aio_write,
@@ -1016,7 +1011,7 @@ class Ioctx(object):
         :type onsafe: completion
 
         :raises: :class:`Error`
-        :returns: completion object 
+        :returns: completion object
         """
         completion = self.__get_completion(oncomplete, onsafe)
         ret = run_in_thread(self.librados.rados_aio_write_full,
@@ -1047,7 +1042,7 @@ class Ioctx(object):
         :type onsafe: completion
 
         :raises: :class:`Error`
-        :returns: completion object 
+        :returns: completion object
         """
         completion = self.__get_completion(oncomplete, onsafe)
         ret = run_in_thread(self.librados.rados_aio_append,
@@ -1090,11 +1085,8 @@ class Ioctx(object):
         :returns: completion object
         """
         buf = create_string_buffer(length)
-        def oncomplete_(completion_v):
-            return_value = completion_v.get_return_value()
-            return oncomplete(completion_v,
-                              ctypes.string_at(buf, return_value) if return_value >= 0 else None)
-
+        def oncomplete_(completion):
+            return oncomplete(completion, buf.value)
         completion = self.__get_completion(oncomplete_, None)
         ret = run_in_thread(self.librados.rados_aio_read,
                             (self.io, c_char_p(object_name),
@@ -1210,7 +1202,7 @@ class Ioctx(object):
 
         :raises: :class:`TypeError`
         :raises: :class:`LogicError`
-        :returns: int - number of bytes written 
+        :returns: int - 0 on success
         """
         self.require_ioctx_open()
         if not isinstance(key, str):
@@ -1275,7 +1267,7 @@ returned %d, but should return zero on success." % (self.name, ret))
 
         :raises: :class:`TypeError`
         :raises: :class:`LogicError`
-        :returns: int - number of bytes written
+        :returns: int - 0 on success
         """
         self.require_ioctx_open()
         if not isinstance(key, str):
@@ -1302,7 +1294,7 @@ returned %d, but should return zero on success." % (self.name, ret))
         :param key: name of the object
         :type key: str
         :param length: the number of bytes to read (default=8192)
-        :type length: int 
+        :type length: int
         :param offset: byte offset in the object to begin reading at
         :type offset: int
 
@@ -1409,7 +1401,7 @@ returned %d, but should return zero on success." % (self.name, ret))
         :raises: :class:`Error`
         :returns: int - 0 on success, otherwise raises error
         """
-        
+
         self.require_ioctx_open()
         if not isinstance(key, str):
             raise TypeError('key must be a string')
@@ -1422,7 +1414,7 @@ returned %d, but should return zero on success." % (self.name, ret))
     def stat(self, key):
         """
         Get object stats (size/mtime)
-        
+
         :param key: the name of the object to get stats from
         :type key: str
 
@@ -1446,7 +1438,7 @@ returned %d, but should return zero on success." % (self.name, ret))
     def get_xattr(self, key, xattr_name):
         """
         Get the value of an extended attribute on an object.
-        
+
         :param key: the name of the object to get xattr from
         :type key: str
         :param xattr_name: which extended attribute to read
@@ -1476,7 +1468,7 @@ returned %d, but should return zero on success." % (self.name, ret))
     def get_xattrs(self, oid):
         """
         Start iterating over xattrs on an object.
-        
+
         :param oid: the name of the object to get xattrs from
         :type key: str
 
@@ -1497,7 +1489,7 @@ returned %d, but should return zero on success." % (self.name, ret))
     def set_xattr(self, key, xattr_name, xattr_value):
         """
         Set an extended attribute on an object.
-        
+
         :param key: the name of the object to set xattr to
         :type key: str
         :param xattr_name: which extended attribute to set
@@ -1526,7 +1518,7 @@ returned %d, but should return zero on success." % (self.name, ret))
     def rm_xattr(self, key, xattr_name):
         """
         Removes an extended attribute on from an object.
-        
+
         :param key: the name of the object to remove xattr from
         :type key: str
         :param xattr_name: which extended attribute to remove
@@ -1549,7 +1541,7 @@ returned %d, but should return zero on success." % (self.name, ret))
         return True
 
     def list_objects(self):
-        """ 
+        """
         Get ObjectIterator on rados.Ioctx object.
 
         :returns: ObjectIterator
@@ -1558,7 +1550,7 @@ returned %d, but should return zero on success." % (self.name, ret))
         return ObjectIterator(self)
 
     def list_snaps(self):
-        """ 
+        """
         Get SnapIterator on rados.Ioctx object.
 
         :returns: SnapIterator
@@ -1716,7 +1708,7 @@ MONITOR_LEVELS = [
    "debug",
    "info",
    "warn", "warning",
-   "err", "error", 
+   "err", "error",
    "sec",
    ]
 
